// The width and height of the captured photo. We will set the
// width to the value defined here, but the height will be
// calculated based on the aspect ratio of the input stream.

var width = 320; // We will scale the photo width to this
var height = 0; // This will be computed based on the input stream

// |streaming| indicates whether or not we're currently streaming
// video from the camera. Obviously, we start at false.

var streaming = false;

// The various HTML elements we need to configure or control. These
// will be set by the startup() function.

var video = null;
var canvas = null;
var photo = null;
var startbutton = null;

function startup() {
	video = document.getElementById("video");
	if (video) {
		canvas = document.getElementById("canvas");
		photo = document.getElementById("photo");
		startbutton = document.getElementById("startbutton");
		navigator.mediaDevices
			.getUserMedia({ video: true, audio: false })
			.then(function (stream) {
				video.srcObject = stream;
				video.play();
			})
			.catch(function (err) {
				console.log("An error occurred: " + err);
			});

		video.addEventListener(
			"canplay",
			function (ev) {
				if (!streaming) {
					height = video.videoHeight / (video.videoWidth / width);

					// Firefox currently has a bug where the height can't be read from
					// the video, so we will make assumptions if this happens.

					if (isNaN(height)) {
						height = width / (4 / 3);
					}

					video.setAttribute("width", width);
					video.setAttribute("height", height);
					canvas.setAttribute("width", width);
					canvas.setAttribute("height", height);
					streaming = true;
				}
			},
			false
		);

		startbutton.addEventListener(
			"click",
			function (ev) {
				recognizeFace();
				ev.preventDefault();
			},
			false
		);

		clearphoto();
	}
}

// Fill the photo with an indication that none has been
// captured.

function clearphoto() {
	var context = canvas.getContext("2d");
	context.fillStyle = "#AAA";
	context.fillRect(0, 0, canvas.width, canvas.height);

	var data = canvas.toDataURL("image/png");
	photo.setAttribute("src", data);
}

function setCanvas(idImg = "", maxDimensionSet = "") {
	//var video = document.getElementById('video');
	var h, w;

	if (idImg) {
		img = document.getElementById(idImg);
		w = img.naturalWidth;
		h = img.naturalHeight;
	} else {
		img = video;
		w = video.videoWidth;
		h = video.videoHeight;
	}

	var canvas = document.getElementById("canvas");
	var context = canvas.getContext("2d");

	var maxDimension = 320;
	if (maxDimensionSet) maxDimension = maxDimensionSet;

	var scale = Math.min(maxDimension / w, maxDimension / h);
	var scaledWidth = w * scale;
	var scaledHeight = h * scale;

	canvas.width = scaledWidth;
	canvas.height = scaledHeight;

	if (h > w) {
		ht = height;
		wt = width;
		height = wt;
		width = ht;
	}
	//console.warn(scaledWidth, scaledHeight)
	context.drawImage(img, 0, 0, scaledWidth, scaledHeight);
	photoData.img = canvas.toDataURL("image/jpeg");
	//console.warn('photoData:', photoData);
	$("#loading").show();
	$("#canvas").show();
	$("#videoContainer").hide();
}
